// slider.cpp
//copyright 2008 Beer[Tsutsumi Hiroyuki]

#include	"slider.h"

slider::slider(BRect frame,const char *name,const char *label,BView *target,float min,float max,float val):BView(frame,name,B_FOLLOW_NONE,B_WILL_DRAW)
{
	fMin=min;	fMax=max;	fValue=val;
	cLabel=label;
	vTarget=target;
	rBar.Set(cLabel?frame.Width()*.25:frame.Width()*.05,frame.Height()/3,frame.Width()-frame.Height(),frame.Height()*2/3);
	setslider();
	drag=false;
	lastval=fValue;
}

void slider::AttachedToWindow()
{
//	SetColorsFromParent();
	SetViewColor(Parent()->ViewColor());
	SetHighColor(Parent()->HighColor());
	SetLowColor(Parent()->LowColor());

	SetFont(be_plain_font);
	SetFontSize(rBar.Height()*1.5);
	if(cLabel)pLabel.Set(rBar.left-StringWidth(cLabel)-3,rBar.Height()*.8);
	pValue.Set(rBar.right+3,rBar.bottom);
}

void slider::Draw(BRect rect)
{
	SetHighColor(0,0,0);
	if(cLabel)DrawString(cLabel,pLabel);
	char s[16];sprintf(s,"%d",(int)fValue);
	DrawString(s,pValue);
	SetHighColor(180,180,180);
	FillRect(rBar);
	SetHighColor(220,220,220);
	FillRect(rSlider);
}

void slider::MouseDown(BPoint pt)
{
	if(rBar.Contains(pt)){
		drag=true;
		slidermoved(pt.x);
	}
}

void slider::MouseUp(BPoint pt)
{
	drag=false;
}

void slider::MouseMoved(BPoint pt, uint32 code, const BMessage *msg)
{
	uint32	buttons;
	GetMouse(&pt,&buttons);
	if(drag&&(buttons&B_PRIMARY_MOUSE_BUTTON)){
		slidermoved(pt.x);
	}
}

void slider::SetValue(float value)
{
	fValue=(value<fMin)?fMin:(value>fMax)?fMax:value;
	setslider();
	Invalidate();
}

float slider::Value()
{
	return(fValue);
}

void slider::setslider()
{
	float size=rBar.Height()-2;
	rSlider.left=(rBar.Width()-size-2)*(fValue-fMin)/(fMax-fMin)+rBar.left+1;
	rSlider.right=rSlider.left+size;
	rSlider.top=rBar.top+1;
	rSlider.bottom=rBar.bottom-1;
}

void slider::slidermoved(int center)
{
	float size=rSlider.Width();
	float val=(center-size/2-rBar.left-1)/(rBar.Width()-size-2)*(fMax-fMin)+fMin;
	SetValue(val);
	if(fValue!=lastval){
		BMessage mes('Smov');
		mes.AddPointer("source",this);
//		mes.AddFloat(Name(),fValue);
		mes.AddFloat("value",fValue);
		BMessenger(vTarget).SendMessage(&mes);	//Window()->PostMessage(&mes,vTarget);
		lastval=fValue;
	}
}
//------------------------
rgbslider::rgbslider(BRect frame,const char *name,const char *label,BView *target,bool alpha,uint32 val):BView(frame,name,B_FOLLOW_NONE,B_WILL_DRAW)
{
	xValue=val;
	cLabel=label;
	vTarget=target;
	bAlpha=alpha;
	float	l=cLabel?frame.Width()*.25:frame.Width()*.05;
	float	r=frame.Width()-frame.Height();
	rBar.Set(l,frame.Height()/3,r,frame.Height()*2/3);
	if(bAlpha){
		rBarr.Set(l,0,r,frame.Height()*.25);
		rBarg.Set(l,frame.Height()*.25,r,frame.Height()*.50);
		rBarb.Set(l,frame.Height()*.50,r,frame.Height()*.75);
		rBara.Set(l,frame.Height()*.75,r,frame.Height());
	}else{
		rBarr.Set(l,0,r,frame.Height()/3);
		rBarg.Set(l,frame.Height()*.33,r,frame.Height()*.66);
		rBarb.Set(l,frame.Height()*.66,r,frame.Height());
	}
	setslider();
	drag=0;
	lastval=xValue;
}

void rgbslider::AttachedToWindow()
{
//	SetColorsFromParent();
	SetViewColor(Parent()->ViewColor());
	SetHighColor(Parent()->HighColor());
	SetLowColor(Parent()->LowColor());

	SetFont(be_plain_font);
	if(cLabel)pLabel.Set(rBar.left-StringWidth(cLabel)-3,rBar.Height()*.8);
	pValuer.Set(rBarr.right+3,rBarr.bottom);
	pValueg.Set(rBarg.right+3,rBarg.bottom);
	pValueb.Set(rBarb.right+3,rBarb.bottom);
	if(bAlpha)pValuea.Set(rBara.right+3,rBara.bottom);
}

void rgbslider::Draw(BRect rect)
{
	SetHighColor(0,0,0);
	if(cLabel){SetFontSize(rBar.Height()*1.5);DrawString(cLabel,pLabel);}
	SetFontSize(rBarr.Height()+1);
	char s[16];
	sprintf(s,"%02x",(uchar)(xValue>>16));DrawString(s,pValuer);
	sprintf(s,"%02x",(uchar)(xValue>>8));DrawString(s,pValueg);
	sprintf(s,"%02x",(uchar)xValue);DrawString(s,pValueb);
	SetHighColor(120,0,0);FillRect(rBarr);
	SetHighColor(0,120,0);FillRect(rBarg);
	SetHighColor(0,0,120);FillRect(rBarb);
	SetHighColor(255,0,0);FillRect(rSliderr);
	SetHighColor(0,255,0);FillRect(rSliderg);
	SetHighColor(0,0,255);FillRect(rSliderb);
	if(bAlpha){
		sprintf(s,"%02x",(uchar)(xValue>>24));DrawString(s,pValuea);
		SetHighColor(150,150,150);FillRect(rBara);
		SetHighColor(255,255,255);FillRect(rSlidera);
	}
}

void rgbslider::MouseDown(BPoint pt)
{
	drag=0;
	if(rBarr.Contains(pt))drag=1;
	if(rBarg.Contains(pt))drag=2;
	if(rBarb.Contains(pt))drag=3;
	if((bAlpha)&&(rBara.Contains(pt)))drag=4;
	if(drag)slidermoved(pt.x);
}

void rgbslider::MouseUp(BPoint pt)
{
	drag=0;
}

void rgbslider::MouseMoved(BPoint pt, uint32 code, const BMessage *msg)
{
	uint32	buttons;
	GetMouse(&pt,&buttons);
	if(drag&&(buttons&B_PRIMARY_MOUSE_BUTTON)){
		slidermoved(pt.x);
	}
}

void rgbslider::SetValue(uint32 value)
{
	if(!bAlpha)value&=0xffffff;
	xValue=value;
	setslider();
	Invalidate();
	lastval=xValue;
}

uint32 rgbslider::Value()
{
	return(xValue);
}

void rgbslider::setslider()
{
	uchar	r,g,b,a;
	a=xValue>>24;r=xValue>>16;g=xValue>>8;b=xValue;
	float size=rBar.Height()-2;
	rSliderr.left=(rBar.Width()-size-2)*r/255+rBarr.left+1;
	rSliderr.right=rSliderr.left+size;
	rSliderr.top=rBarr.top+1;
	rSliderr.bottom=rBarr.bottom-1;
	rSliderg.left=(rBar.Width()-size-2)*g/255+rBarg.left+1;
	rSliderg.right=rSliderg.left+size;
	rSliderg.top=rBarg.top+1;
	rSliderg.bottom=rBarg.bottom-1;
	rSliderb.left=(rBar.Width()-size-2)*b/255+rBarb.left+1;
	rSliderb.right=rSliderb.left+size;
	rSliderb.top=rBarb.top+1;
	rSliderb.bottom=rBarb.bottom-1;
	if(bAlpha){
		rSlidera.left=(rBar.Width()-size-2)*a/255+rBara.left+1;
		rSlidera.right=rSlidera.left+size;
		rSlidera.top=rBara.top+1;
		rSlidera.bottom=rBara.bottom-1;
	}
}

void rgbslider::slidermoved(int center)
{
	float size=rSliderr.Width();
	float	v=0;
	if(drag==1)v=(center-size/2-rBarr.left-1)/(rBarr.Width()-size-2)*255;
	if(drag==2)v=(center-size/2-rBarg.left-1)/(rBarg.Width()-size-2)*255;
	if(drag==3)v=(center-size/2-rBarb.left-1)/(rBarb.Width()-size-2)*255;
	if(drag==4)v=(center-size/2-rBara.left-1)/(rBara.Width()-size-2)*255;
	if(v<0)v=0;if(v>255)v=255;
	uint32 val=0;
	if(drag==1)val=lastval&0xff00ffff|((uchar)v)<<16;
	if(drag==2)val=lastval&0xffff00ff|((uchar)v)<<8;
	if(drag==3)val=lastval&0xffffff00|((uchar)v);
	if(drag==4)val=lastval&0x00ffffff|((uchar)v)<<24;
	if(!bAlpha)val&=0xffffff;
	if(val!=lastval){
		BMessage mes('Smov');
		mes.AddPointer("source",this);
//		mes.AddFloat(Name(),val);
		mes.AddInt32("argb",val);
		BMessenger(vTarget).SendMessage(&mes);	//Window()->PostMessage(&mes,vTarget);
//		lastval=xValue;
		SetValue(val);
	}
}
